(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs/add/operator/do'), require('lodash'), require('@angular/common/http'), require('rxjs/Observable'), require('rxjs/add/observable/of')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', 'rxjs/add/operator/do', 'lodash', '@angular/common/http', 'rxjs/Observable', 'rxjs/add/observable/of'], factory) :
	(factory((global.ng = global.ng || {}, global.ng.jsonapi = {}),global.ng.core,global.Rx.Observable.prototype,global.lodash,global.ng.common.http,global.Rx));
}(this, (function (exports,core,_do,lodash,http,Observable) { 'use strict';

"use strict";
var Observable_1 = require('../../Observable');
var map_1 = require('../../operator/map');
Observable_1.Observable.prototype.map = map_1.map;

"use strict";
var Observable_1$1 = require('../../Observable');
var catch_1 = require('../../operator/catch');
Observable_1$1.Observable.prototype.catch = catch_1._catch;
Observable_1$1.Observable.prototype._catch = catch_1._catch;

/**
 * @license jsonapi
 * MIT license
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * `ApiConfig` represents the configuration of the api.
 */
var ApiConfig = (function () {
    function ApiConfig() {
    }
    return ApiConfig;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Config
var ApiInterceptor = (function () {
    function ApiInterceptor(config) {
        this.config = config;
    }
    /**
     * Intercept http request to add api url path.
     *
     * @param {?} req Http request
     * @param {?} next Http handler
     * @return {?}
     */
    ApiInterceptor.prototype.intercept = /**
     * Intercept http request to add api url path.
     *
     * @param {?} req Http request
     * @param {?} next Http handler
     * @return {?}
     */
    function (req, next) {
        var /** @type {?} */ headers = req.headers;
        // Check headers
        if (this.config.headers) {
            lodash.forOwn(this.config.headers, function (value, key) {
                headers = headers.set(key, value);
            });
        }
        // Change api url
        var /** @type {?} */ apiReq = req.clone({
            headers: headers,
            url: this.getApiUrl() + req.url,
        });
        // Pass on the cloned request instead of the original request.
        return next.handle(apiReq);
    };
    /**
     * Generate api url with version.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com
     * @return {?}
     */
    ApiInterceptor.prototype.getApiUrl = /**
     * Generate api url with version.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com
     * @return {?}
     */
    function () {
        return this.config.url;
    };
    ApiInterceptor.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    ApiInterceptor.ctorParameters = function () { return [
        { type: ApiConfig, },
    ]; };
    return ApiInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
/**
 * `ResponseConverter` convert backend response to camelCase.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
var ResponseConverter = (function () {
    function ResponseConverter() {
    }
    /**
     * Intercept http request to convert backend response to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?}
     */
    ResponseConverter.prototype.intercept = /**
     * Intercept http request to convert backend response to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?}
     */
    function (req, next) {
        var _this = this;
        return next.handle(req).do(function (event) {
            if (event instanceof http.HttpResponse && event.body) {
                if (event.body.data) {
                    event.body.data = _this.convert(event.body.data);
                }
                if (event.body.meta) {
                    event.body.meta = _this.convert(event.body.meta);
                }
            }
        }, function (event) {
            if (event instanceof http.HttpErrorResponse) {
                if (event.error.errors) {
                    event.error.errors = _this.convert(event.error.errors);
                }
            }
        });
    };
    /**
     * Convert object or array to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} data Object/array to convert
     * @return {?} Result converted
     */
    ResponseConverter.prototype.convert = /**
     * Convert object or array to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} data Object/array to convert
     * @return {?} Result converted
     */
    function (data) {
        var _this = this;
        // Check if data is array, convert each item
        if (lodash.isArray(data)) {
            var /** @type {?} */ response_1 = [];
            // Convert each item
            data.forEach(function (item) {
                response_1.push(_this.convert(item));
            });
            // Return new reponse
            return response_1;
        }
        // Check if data is a object, convert each key to camel case and check if value is an array or not
        if (lodash.isObject(data)) {
            var /** @type {?} */ response_2 = {};
            // Convert each property key and value
            lodash.forOwn(data, function (value, key) {
                response_2[_this.convertKey(key)] = _this.convert(value);
            });
            // Return new response
            return response_2;
        }
        return data;
    };
    /**
     * Convert string to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} key String to convert
     * @return {?} String converted
     */
    ResponseConverter.prototype.convertKey = /**
     * Convert string to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} key String to convert
     * @return {?} String converted
     */
    function (key) {
        var /** @type {?} */ keySplitted = key.split('.');
        return lodash.map(keySplitted, function (k) { return lodash.camelCase('' + k); }).join('.');
    };
    ResponseConverter.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    ResponseConverter.ctorParameters = function () { return []; };
    return ResponseConverter;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
/**
 * `RequestConverter` convert backend request body to snakeCase.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
var RequestConverter = (function () {
    function RequestConverter() {
    }
    /**
     * Intercept http request to convert backend request body to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?} Response
     */
    RequestConverter.prototype.intercept = /**
     * Intercept http request to convert backend request body to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?} Response
     */
    function (req, next) {
        var _this = this;
        var /** @type {?} */ reqBody = req.body;
        var /** @type {?} */ params = req.params;
        var /** @type {?} */ newReq;
        // Check if request body is not empty
        if (reqBody) {
            reqBody = this.convert(reqBody);
        }
        // Check query params
        if (params) {
            var /** @type {?} */ newParams_1 = new http.HttpParams();
            // Parse each param key
            params.keys().map(function (key) {
                newParams_1 = newParams_1.set(_this.convertKey(key), '' + params.get(key));
            });
            // Set new params
            params = newParams_1;
        }
        // Clone request
        newReq = req.clone({ body: reqBody, params: params });
        // Make request
        return next.handle(newReq);
    };
    /**
     * Convert object or array keys to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} data Object/array to convert
     * @return {?} Result converted
     */
    RequestConverter.prototype.convert = /**
     * Convert object or array keys to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} data Object/array to convert
     * @return {?} Result converted
     */
    function (data) {
        var _this = this;
        // Check if data is array, convert each item
        if (lodash.isArray(data)) {
            var /** @type {?} */ response_1 = [];
            // Convert each item
            data.forEach(function (item) {
                response_1.push(_this.convert(item));
            });
            // Return new reponse
            return response_1;
        }
        // Check if data is a object, convert each key to snake case and check if value is an array or not
        if (lodash.isObject(data)) {
            var /** @type {?} */ response_2 = {};
            // Convert each property key and value
            lodash.forOwn(data, function (value, key) {
                response_2[_this.convertKey(key)] = _this.convert(value);
            });
            // Return new response
            return response_2;
        }
        return data;
    };
    /**
     * Convert string to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} key String to convert
     * @return {?} String converted
     */
    RequestConverter.prototype.convertKey = /**
     * Convert string to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} key String to convert
     * @return {?} String converted
     */
    function (key) {
        var /** @type {?} */ keyConverted = key.indexOf('_') >= 0 ? key : lodash.snakeCase('' + key);
        return key.endsWith('[]') ? keyConverted + '[]' : keyConverted;
    };
    RequestConverter.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    RequestConverter.ctorParameters = function () { return []; };
    return RequestConverter;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
/**
 * `HttpCacheService` service to cache requests.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 * @export
 */
var HttpCacheService = (function () {
    function HttpCacheService() {
        this.requests = [];
        this.responses = [];
    }
    /**
     * Returns a cached response, if any, or null if not present.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req
     * @return {?}
     */
    HttpCacheService.prototype.get = /**
     * Returns a cached response, if any, or null if not present.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req
     * @return {?}
     */
    function (req) {
        var /** @type {?} */ reqIndex = this.findRequest(req);
        var /** @type {?} */ resp = null;
        // Find request and response
        if (reqIndex >= 0) {
            resp = this.responses[reqIndex];
        }
        return resp;
    };
    /**
     * Adds or updates the response in the cache.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req Request
     * @param {?} resp Response
     * @return {?}
     */
    HttpCacheService.prototype.put = /**
     * Adds or updates the response in the cache.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req Request
     * @param {?} resp Response
     * @return {?}
     */
    function (req, resp) {
        // Push data
        this.requests.push(req);
        this.responses.push(resp);
    };
    /**
     * Find saved http request.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req Request to compare
     * @return {?} Response index
     */
    HttpCacheService.prototype.findRequest = /**
     * Find saved http request.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req Request to compare
     * @return {?} Response index
     */
    function (req) {
        // Find http request
        return lodash.findIndex(this.requests, function (savedReq) {
            return savedReq.url === req.url;
        });
    };
    HttpCacheService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    HttpCacheService.ctorParameters = function () { return []; };
    return HttpCacheService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Services
var CACHE_KEY = 'cache';
/**
 * `CachingInterceptor` check cache param and cache request.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
var CachingInterceptor = (function () {
    // Public methods
    /**
     * Creates an instance of CachingInterceptor.
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     * @param cache Service to cache request
     * @memberof CachingInterceptor
     */
    function CachingInterceptor(cache) {
        this.cache = cache;
    }
    /**
     * Intercept http request to convert backend request body to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof CachingInterceptor
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?} Response
     */
    CachingInterceptor.prototype.intercept = /**
     * Intercept http request to convert backend request body to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof CachingInterceptor
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?} Response
     */
    function (req, next) {
        var _this = this;
        // Before doing anything, it's important to only cache GET requests.
        // Skip this interceptor if the request method isn't GET.
        if (req.method !== 'GET' || !this.cacheRequest(req)) {
            return next.handle(req);
        }
        // First, check the cache to see if this request exists.
        var /** @type {?} */ cachedResponse = this.cache.get(req);
        if (cachedResponse) {
            // A cached response exists. Serve it instead of forwarding
            // the request to the next handler.
            return Observable.Observable.of(cachedResponse);
        }
        // Delete cache key params
        var /** @type {?} */ newReq = req.clone({ params: req.params.delete(CACHE_KEY) });
        // No cached response exists. Go to the network, and cache
        // the response when it arrives.
        return next.handle(newReq).do(function (event) {
            // Remember, there may be other events besides just the response.
            if (event instanceof http.HttpResponse) {
                // Update the cache.
                // Update the cache.
                _this.cache.put(newReq, event);
            }
        });
    };
    /**
     * Check if request should be cached
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof CachingInterceptor
     * @param {?} req Request
     * @return {?} True if should be cached
     */
    CachingInterceptor.prototype.cacheRequest = /**
     * Check if request should be cached
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof CachingInterceptor
     * @param {?} req Request
     * @return {?} True if should be cached
     */
    function (req) {
        return req.params && req.params.get(CACHE_KEY) === 'true';
    };
    CachingInterceptor.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    CachingInterceptor.ctorParameters = function () { return [
        { type: HttpCacheService, },
    ]; };
    return CachingInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Interfaces
/**
 * `BaseService` implements the basic REST interaction of any service with the api.
 *
 * \@typeparam R Represents the resource to be handled.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 * @abstract
 */
var BaseService = (function () {
    /**
     * Creates a new `BaseService` instance.
     * @param httpService Http service to make requests.
     * @param rName Name of resource.
     * @param rNameSingular Singular name of resource.
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     */
    function BaseService(httpService, config, rName) {
        this.httpService = httpService;
        this.config = config;
        this.rName = rName;
        this.resourceName = rName;
    }
    /**
     * Get a list of `R` objects
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?=} options Optional parameters
     * @return {?} An promise with an array of `R` objects.
     *
     */
    BaseService.prototype.get = /**
     * Get a list of `R` objects
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?=} options Optional parameters
     * @return {?} An promise with an array of `R` objects.
     *
     */
    function (options) {
        return this.httpService.get(this.getPath(), options);
    };
    /**
     * Get one `R` object
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?} id Resource id
     * @param {?=} options
     * @return {?} An promise with an array of `R` objects.
     *
     */
    BaseService.prototype.getOne = /**
     * Get one `R` object
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?} id Resource id
     * @param {?=} options
     * @return {?} An promise with an array of `R` objects.
     *
     */
    function (id, options) {
        return this.httpService.get(this.getPath(id), options);
    };
    /**
     * Create one `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    BaseService.prototype.post = /**
     * Create one `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    function (data, options) {
        return this.httpService.post(this.getPath(), data, options);
    };
    /**
     * Update one `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    BaseService.prototype.put = /**
     * Update one `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    function (data, options) {
        return this.httpService.put(this.getPath(data.id), data, options);
    };
    /**
     * Delete on `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    BaseService.prototype.delete = /**
     * Delete on `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    function (data, options) {
        return this.httpService.delete(this.getPath(data.id), options);
    };
    /**
     * Get current path
     * @param id? Resource id
     * @returns String with current path.
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     */
    /**
     * Get current path
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?=} id
     * @return {?} String with current path.
     *
     */
    BaseService.prototype.getPath = /**
     * Get current path
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?=} id
     * @return {?} String with current path.
     *
     */
    function (id) {
        var /** @type {?} */ path = '';
        var /** @type {?} */ prefix = this.config && this.config.prefix ? this.config.prefix : '';
        // Add prefix to api path
        if (prefix !== '') {
            path += '/' + prefix;
        }
        // Add version to api path
        var /** @type {?} */ version = this.config && this.config.version ? this.config.version : 0;
        if (version > 0) {
            path += '/v' + version;
        }
        // Add resource name
        path += '/' + this.resourceName;
        // Check if should add id
        return id && id > 0 ? path + '/' + id : path;
    };
    BaseService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    BaseService.ctorParameters = function () { return [
        { type: http.HttpClient, },
        { type: ApiConfig, },
        null,
    ]; };
    return BaseService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Config
// Interceptors
/**
 * `JsonapiModule`
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
var JsonapiModule = (function () {
    function JsonapiModule() {
    }
    /**
     * @param {?} config
     * @return {?}
     */
    JsonapiModule.forRoot = /**
     * @param {?} config
     * @return {?}
     */
    function (config) {
        return {
            ngModule: JsonapiModule,
            providers: [
                HttpCacheService,
                {
                    provide: ApiConfig,
                    useValue: config
                },
                { provide: http.HTTP_INTERCEPTORS, useClass: ApiInterceptor, multi: true },
                { provide: http.HTTP_INTERCEPTORS, useClass: ResponseConverter, multi: true },
                { provide: http.HTTP_INTERCEPTORS, useClass: RequestConverter, multi: true },
                { provide: http.HTTP_INTERCEPTORS, useClass: CachingInterceptor, multi: true }
            ]
        };
    };
    JsonapiModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [],
                    imports: [
                        http.HttpClientModule
                    ]
                },] },
    ];
    /** @nocollapse */
    JsonapiModule.ctorParameters = function () { return []; };
    return JsonapiModule;
}());

exports.ApiConfig = ApiConfig;
exports.ApiInterceptor = ApiInterceptor;
exports.ResponseConverter = ResponseConverter;
exports.RequestConverter = RequestConverter;
exports.CACHE_KEY = CACHE_KEY;
exports.CachingInterceptor = CachingInterceptor;
exports.BaseService = BaseService;
exports.HttpCacheService = HttpCacheService;
exports.JsonapiModule = JsonapiModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=jsonapi.umd.js.map
