import { HttpInterceptor, HttpHandler, HttpRequest, HttpEvent } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/observable/of';
import 'rxjs/add/operator/do';
import { HttpCacheService } from '../services/http-cache.service';
export declare const CACHE_KEY = "cache";
/**
 * `CachingInterceptor` check cache param and cache request.
 *
 * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
 */
export declare class CachingInterceptor implements HttpInterceptor {
    private cache;
    /**
     * Creates an instance of CachingInterceptor.
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     * @param cache Service to cache request
     * @memberof CachingInterceptor
     */
    constructor(cache: HttpCacheService);
    /**
     * Intercept http request to convert backend request body to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     * @param req Request headers
     * @param next Request handler
     * @returns Response
     * @memberof CachingInterceptor
     */
    intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>>;
    /**
     * Check if request should be cached
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     * @param req Request
     * @returns True if should be cached
     * @memberof CachingInterceptor
     */
    private cacheRequest(req);
}
